#include "precomp.h"

int WINAPI _tWinMain( HINSTANCE hinst, HINSTANCE, LPTSTR, int nCmdShow )
{
   CApp app( hinst );

   return app.Main( nCmdShow );
}

BOOL CALLBACK AboutDlgProc(
   HWND   hdlg,
   UINT   uMsg,
   WPARAM wParam,
   LPARAM lParam )
{
   CApp *pthis = (CApp *) GetProp( hdlg, THIS_PROP );

   if ( !pthis )
   {
      if ( uMsg != WM_INITDIALOG )
         return FALSE;

      pthis = (CApp *) lParam;

      SetProp( hdlg, THIS_PROP, (HANDLE) pthis );
   }

   BOOL fResult = pthis->AboutDlgProc( hdlg, uMsg, wParam, lParam );

   if ( uMsg == WM_DESTROY )
      RemoveProp( hdlg, THIS_PROP );

   return fResult;
}

BOOL CApp::AboutDlgProc(
   HWND   hdlg,
   UINT   uMsg,
   WPARAM wParam,
   LPARAM lParam )
{
   switch ( uMsg )
   {
      case WM_INITDIALOG:

         SetDlgItemText( hdlg, IDC_WEBCTRL, m_szHTML );

         FORWARD_WM_NEXTDLGCTL( hdlg, GetDlgItem( hdlg, IDOK ), 1, PostMessage );

         break;

      case WM_COMMAND:

         switch ( GET_WM_COMMAND_ID( wParam, lParam ) )
         {
            case IDOK:
            case IDCANCEL:

               EndDialog( hdlg, 0 );

               break;

            default:

               return FALSE;
         }

         break;

      default:

         return FALSE;
   }

   return TRUE;
}

LRESULT CALLBACK WndProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   CApp *pthis = (CApp *) GetProp( hwnd, THIS_PROP );

   if ( !pthis )
   {
      if ( uMsg != WM_CREATE )
         return DefWindowProc( hwnd, uMsg, wParam, lParam );

      CREATESTRUCT *pcs = (CREATESTRUCT *) lParam;

      pthis = (CApp *) pcs->lpCreateParams; 

      pthis->m_hwnd = hwnd;
   }

   return pthis->WndProc( uMsg, wParam, lParam );
}

LRESULT CApp::WndProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   LRESULT lResult = 0;

   switch ( uMsg )
   {
      case WM_CREATE:

         SetProp( m_hwnd, THIS_PROP, (HANDLE) this );

         break;

      case WM_SIZE:

         MoveWindow( m_hwndWebCtrl, 0, 0, LOWORD(lParam), HIWORD(lParam), FALSE );

         break;

      case WM_COMMAND:

         switch ( GET_WM_COMMAND_ID( wParam, lParam ) )
         {
            case ID_FILE_EXIT:

               PostMessage( m_hwnd, WM_CLOSE, 0, 0L );

               break;

            case ID_HELP_ABOUT:

               DialogBoxParam( m_hinst, MAKEINTRESOURCE( IDD_ABOUT ), m_hwnd, ::AboutDlgProc, (LPARAM) this );

               break;

            default:

               lResult = DefWindowProc( m_hwnd, uMsg, wParam, lParam );

               break;
         }

         break;

      case WM_DESTROY:

         RemoveProp( m_hwnd, THIS_PROP );

         PostQuitMessage( 0 );

         break;

      default:

         lResult = DefWindowProc( m_hwnd, uMsg, wParam, lParam );

         break;
   }

   return lResult;
}

int CApp::Main( int nCmdShow )
{
   WNDCLASS wc;

   memset( &wc, 0, sizeof(wc) );

   wc.style         = CS_DBLCLKS;
   wc.lpfnWndProc   = ::WndProc;
   wc.hInstance     = m_hinst;
   wc.hIcon         = LoadIcon( m_hinst, MAKEINTRESOURCE( IDR_MODULE ) );
   wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
   wc.lpszMenuName  = MAKEINTRESOURCE( IDR_MODULE );
   wc.lpszClassName = _T("WebCtrlHost");

   RegisterClass( &wc );

   m_hwnd = CreateWindow(
      wc.lpszClassName,
      _T("Web Control Host"),
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT,
      0,
      CW_USEDEFAULT,
      0,
      NULL,
      NULL,
      m_hinst,
      this );

#ifdef WEBCTRL_AUTOINIT

   m_hwndWebCtrl = CreateWebControl(
      m_szHTML,
      WS_CHILD | WS_VISIBLE,
      0, 0, 0, 0,
      m_hwnd,
      IDC_WEBCTRL );

#else

   InitWebCtrl();

   m_hwndWebCtrl = CreateWindow(
      WC_WEBCTRL,
      m_szHTML,
      WS_CHILD | WS_VISIBLE,
      0, 0, 0, 0,
      m_hwnd,
      (HMENU) IDC_WEBCTRL,
      m_hinst,
      NULL );

#endif

   ShowWindow( m_hwnd, nCmdShow );
   UpdateWindow( m_hwnd );

   MSG msg;

   while ( GetMessage( &msg, NULL, 0, 0 ) )
   {
      TranslateMessage( &msg );
      DispatchMessage( &msg );
   }

#ifndef WEBCTRL_AUTOINIT

   UninitWebCtrl();

#endif

   return msg.wParam;
}

CApp::CApp( HINSTANCE hinst )
{
   m_hinst = hinst;

   TCHAR szModule[MAX_PATH];

   GetModuleFileName( m_hinst, szModule, NUMCHARS(szModule) );

   wsprintf( m_szHTML, _T("res://%s/index.htm"), szModule );
}
